
#dependency

library(dplyr,quietly = T,warn.conflicts = T)
library(ggplot2,quietly = T)

hash <- function(key,value){
  res <- as.vector(value)
  names(res) <- as.character(key)
  return (res)
}

mytime <- function(expression) {
  t1 <- proc.time()
  res <- eval(expression)
  print(proc.time() - t1)
  res
}

runparallel <- function(objlist, FUN, cores = 4, ...) {
  
  cl <- parallel::makeCluster(cores)
  res <- parallel::parLapply(cl, X = objlist, fun = FUN, ...)
  parallel::stopCluster(cl)
  
  invisible(res)
}

call_intra_matrix <- function(hybfile,removeOverlap=F,Bait_RNA="NC_045512-2_SARS-CoV-2_virusRNA",matrxfile,...){
  library(dplyr)
  hyb <- data.table::fread(hybfile,sep = "\t",select = 1:15)
 
  intra.hyb <- hyb %>% filter(V4 == V10) %>% filter(V4 == Bait_RNA) %>% select(c(7,8,13,14,1)) 
  
  intra.hyb <- intra.hyb %>%  tidyr::separate(col = "V1",into=c("id","count"),sep="_") %>% type.convert(as.is=F)
  
  
  if(removeOverlap){
    intra.hyb <-  intra.hyb %>% mutate(overlap = 1+pmin(V8,V14)-pmax(V7,V13)) %>% dplyr::filter(overlap < 0)
  }
  # intra 
  Contact_matrix <- HybToMatrix(Hyb = intra.hyb,filename = matrxfile)
}

# from hyb file to contact matrix core function
hyb2CM <- function(hyblines, binsize = 10,bins=3300,dedup=T,...) {
  library(dplyr)
  hyblines.bin <- floor(hyblines[,1:4] / binsize) # as.integer only supports vector
  Arm5=(0:bins)*binsize
  Arm3=(0:bins)*binsize
  matrix <- matrix(data = 0,nrow = 1+bins,ncol = 1+bins,dimnames=list(Arm5,Arm3))
  for(i in 1:nrow(hyblines)){
    x <- ifelse(dedup,1,as.numeric(hyblines[i,6]))
    matrix[hyblines.bin[i,1]:hyblines.bin[i,2],hyblines.bin[i,3]:hyblines.bin[i,4]] <- matrix[hyblines.bin[i,1]:hyblines.bin[i,2],hyblines.bin[i,3]:hyblines.bin[i,4]]  + x
      
  }
  data <- reshape2::melt(matrix, varnames = c("Arm5", "Arm3"),value.name="n") %>% dplyr::filter(n>0)
  
}

# from hyb to contact matrix file function
HybToMatrix <- function(Hyb,filename,...){
  cores <- parallel::detectCores() -1
  idxes <- sort(sample(letters[1:cores], nrow(Hyb), replace = TRUE))
  HybList <- split(Hyb, idxes)
  result <- mytime({
    runparallel(HybList, FUN = hyb2CM, cores = cores)
  }) %>% bind_rows()  %>% group_by(Arm5,Arm3) %>% summarise(n = sum(n)) %>% as.data.frame()
  data.table::fwrite(result,file = filename,row.names = F,sep = "\t")
  # return(result)
}





Select_matrix2cdt <- function(matrix,contrast = 10,cdtFileName,selectedRegion=c(1,100), val="n",cutoff=0,...){
  levels <- (floor(selectedRegion[2]/10):floor(selectedRegion[1]/10))*10
  matrix <-  matrix %>% filter(( Arm5 >= selectedRegion[1] & Arm5 <= selectedRegion[2] ) & (Arm3 >= selectedRegion[1] & Arm3 <= selectedRegion[2] )) 
  
  matrix <- subset.data.frame(x = matrix,subset = matrix[[val]] > cutoff)
  
  data.table <- data.table::data.table(bin1=factor(matrix$Arm5,levels = rev(levels)),
                                       bin2=factor(matrix$Arm3,levels = levels),
                                       n   =matrix[[val]]/contrast) 
  
  d <- data.table::dcast.data.table(data =data.table,formula = bin2~bin1,fill = 0,drop = F,value.var="n")
  
  d1 <- cbind(data.frame(COL_1 =d$bin2,COL_2=d$bin2,GWEIGHT=1),d[,2:ncol(d)]) %>% type.convert(as.is=F)
  d2 <- rbind(c("EWEIGHT",rep(1,ncol(d1)-1)),d1)
  
  data.table::fwrite(d2,file=cdtFileName,sep = "\t",quote = F)                                 
}


call_intra_matrix_strigent <- function(hybfile,removeOverlap=F,cutoff_3,Bait_RNA="NC_045512-2_SARS-CoV-2_virusRNA",matrxfile){
  hyb <- data.table::fread(hybfile,sep = "\t",select = 1:15)
  library(dplyr)
  intra.hyb <- hyb %>% filter(V4 == V10) %>% filter(V4 == Bait_RNA) %>% select(c(7,8,13,14))
  if(removeOverlap){
    intra.hyb <-  intra.hyb %>% mutate(overlap = 1+pmin(V8,V14)-pmax(V7,V13)) %>% dplyr::filter(overlap < 0)
  }
  intra.hyb <-  intra.hyb %>% dplyr::filter((V7 < V13) |(overlap < -cutoff_3))
  
  # intra 
  Contact_matrix <- HybToMatrix(Hyb = intra.hyb,filename = matrxfile)
}

call_local35_matrix <- function(hybfile,distance=100,Bait_RNA="NC_045512-2_SARS-CoV-2_virusRNA",matrxfile){
  hyb <- data.table::fread(hybfile,sep = "\t",select = 1:15)
  library(dplyr)
  intra.hyb <- hyb %>% filter(V4 == V10) %>% filter(V4 == Bait_RNA) %>% select(c(7,8,13,14))
  intra.hyb <-  intra.hyb %>% mutate(overlap = 1+pmin(V8,V14)-pmax(V7,V13)) %>% dplyr::filter((V7 > V13) &(overlap > -distance))
  
  # intra 
  Contact_matrix <- HybToMatrix(Hyb = intra.hyb,filename = matrxfile)
}

class_stats_hyb <- function(hybfile,Bait_RNA="NC_045512-2_SARS-CoV-2_virusRNA",breaks,...){
  library(dplyr)
  library(ggplot2)
  hyb <- data.table::fread(hybfile,sep = "\t",select = 1:15)
  intra.hyb <- hyb %>% filter(V4 == V10) %>% filter(V4 == Bait_RNA) %>% select(c(7,8,13,14)) %>% mutate(overlap = 1+pmin(V8,V14)-pmax(V7,V13)) 
  overlap_hist <- intra.hyb %>% count(overlap)
  overlap_hist <- overlap_hist %>% mutate(class = cut(overlap,breaks = breaks,labels = c("long range","distal","proximal","overlap"))) %>% group_by(class) %>% summarise(n=sum(n))
  
}



filter_intra.hyb <- function(hybfile,Bait_RNA="NC_045512-2_SARS-CoV-2_virusRNA",stringent=F,...){
  library(dplyr)
  library(ggplot2)
  hyb <- data.table::fread(hybfile,sep = "\t",select = 1:15)
  intra.hyb <- hyb %>% filter(V4 == V10) %>% filter(V4 == Bait_RNA) %>% mutate(overlap = 1+pmin(V8,V14)-pmax(V7,V13)) 
  intra.hyb <- intra.hyb %>% filter(overlap <0)
  if(stringent){
    intra.hyb <-  intra.hyb %>% dplyr::filter((V7 < V13) |(overlap < -100))
  }
  return(intra.hyb)
}


stats_range_oritation_hyb <- function(hybfile,Bait_RNA="NC_045512-2_SARS-CoV-2_virusRNA",breaks=c(-30000,-1000,-100,0,200),labels=c("long range","distal","proximal","overlap"),...){
  library(dplyr)
  library(ggplot2)
  hyb <- data.table::fread(hybfile,sep = "\t",select = 1:15)

  intra.hyb <- hyb %>% filter(V4 == V10) %>% filter(V4 == Bait_RNA) %>% select(c(7,8,13,14)) %>% mutate(overlap = 1+pmin(V8,V14)-pmax(V7,V13)) 
  
  intra.hyb <- intra.hyb %>% mutate(orientation= ifelse(overlap > 0,ifelse(V8>=V14 & V7 >= V13, "Circ","Dimmer"),ifelse(V7< V13, "5'-3'","3'-5'")))

  overlap_hist <- intra.hyb %>% mutate(class = cut(overlap,breaks = breaks,labels = labels)) %>% group_by(class,orientation) %>% summarise(n=n())
}

stats_range_oritation_hyb2 <- function(hybfile,Bait_RNA="NC_045512-2_SARS-CoV-2_virusRNA",breaks=c(-30000,-1000,-100,0,200),labels=c("long range","distal","proximal","overlap"),...){
  library(dplyr)
  library(ggplot2)
  hyb <- data.table::fread(hybfile,sep = "\t",select = 1:15)
  
  intra.hyb <- hyb %>% filter(V4 == V10) %>% filter(V4 == Bait_RNA) %>% select(c(7,8,13,14)) %>% mutate(overlap = 1+pmin(V8,V14)-pmax(V7,V13)) 
  # intra.hyb <- hyb %>% filter(V4 == V10) %>% filter(V4 == Bait_RNA) %>% mutate(overlap = 1+pmin(V8,V14)-pmax(V7,V13)) 
  intra.hyb <- intra.hyb %>% mutate(orientation= ifelse(V7< V13, "5'-3'","3'-5'"))
  
  intra.all <- nrow(intra.hyb)
  Count_orientation <- intra.hyb %>% count(orientation)
  overlap_hist <- intra.hyb %>% mutate(class = cut(overlap,breaks = breaks,labels = labels)) %>% group_by(class,orientation) %>% summarise(n=n())
}

# for a given samplelist, calculate distance and orientation distribution, then ggplot 
stat_distance_orientation <- function(samplelist,hybdir="../hyb_R2/hyb/",suffix= "_comp_SARS-CoV-2_no_polyA_hybrids_ua.hyb.gz",breaks = breaks,labels = labels,strait=T,...){
  
  stats_range_df <- lapply(names(samplelist), function(s){
    
    x <- samplelist[[s]]
    hybfile <- paste0(hybdir,x,suffix)
    
    stats_range_oritation_x <- stats_range_oritation_hyb2(hybfile = hybfile,Bait_RNA = "NC_045512-2_SARS-CoV-2_virusRNA",breaks = breaks,labels = labels) %>% as.data.frame() %>% mutate(sample=s)
  }) %>% bind_rows() %>% as.data.frame()
    maxcount <- stats_range_df %>% group_by(class,sample) %>% summarise(n = sum(n)) 
    maxcount <- max(maxcount$n)
  
  p_list <- lapply(names(samplelist), function(x){
    gdata <- stats_range_df %>% filter(sample == x)
    
    p <- ggplot(gdata,aes(class,n))+
      geom_bar(aes(fill=orientation),stat = "identity")+
      scale_fill_manual(values =col_orientation )+
    #  scale_y_continuous(limits = c(0,maxcount))+
      labs(title = x,y="chimeric reads counts")+
      theme_classic()+theme(axis.text.x  = element_text(angle = 60,hjust = 1))
    
    if(strait){
      p<- p+scale_y_continuous(limits = c(0,maxcount))
    }
    return(p)
  })
  
  
}


plot_Hyb_junction <- function(x,hybdir="../hyb_PEAR/hyb/",hybsuffix=".NC_045512.hyb.gz",
                              region1=c(0,500),region2=c(0,500),mol="NC_045512-2_SARS-CoV-2_virusRNA"){
  data <- data.table::fread(paste0(hybdir,x,hybsuffix),select = c(4,7,8,10,13,14)) %>% 
    filter(V4==mol & V10==mol) %>%
    filter((V7 <= region1[2] & V8 >= region1[1] & V13 <= region2[2] & V14 >= region2[1])|
    (V7 <= region2[2] & V8 >= region2[1] & V13 <= region1[2] & V14 >= region1[1])) %>% 
    count(V7,V8,V13,V14) 
  
  gdata <- data 
  gdata <-gdata %>% type.convert() %>% mutate(orientation = ifelse(V7<=V13,"5'-3'","3'-5'")) 
  
  
  gdata.53 <- gdata %>% filter(V7 <= V13)  %>%
    data.table::setorder(V14,-V7) %>%
    tibble::rowid_to_column(var = "id") %>% type.convert()

    
  gdata.35 <- gdata %>% filter(V7 >= V13)  %>%
    data.table::setorder(V14,-V7) %>%
    tibble::rowid_to_column(var = "id") %>% type.convert()
  
  
  min_x.1 <- min(gdata.53$V7,gdata.35$V13)
  max_x.1 <- max(gdata.53$V8,gdata.35$V14)
  
  min_x.2 <- min(gdata.53$V13,gdata.35$V7)
  max_x.2 <- max(gdata.53$V14,gdata.53$V8)
  
  p1 <- ggplot(gdata.53)+
    geom_rect(aes(xmin=V7, xmax=V8, ymin=id-1 ,ymax=id),fill="#6699cc")+
   # geom_rect(aes(xmin=V13, xmax=V14, ymin=id-1 ,ymax=id),color="#c1272d")+
    scale_x_continuous(limits = c(min_x.1,max_x.1))+
    labs(title = paste0(x,"  5'-3' chimera"),x="SARS-CoV-2",y="")+
    theme_classic()+theme(axis.text.y = element_blank(),axis.ticks.y = element_blank())
  
  p2 <- ggplot(gdata.53)+
    #geom_rect(aes(xmin=V7, xmax=V8, ymin=id-1 ,ymax=id),color="#0071bc")+
     geom_rect(aes(xmin=V13, xmax=V14, ymin=id-1 ,ymax=id),fill="#cc6666")+
    scale_x_continuous(limits = c(min_x.2,max_x.2))+
    labs(x="SARS-CoV-2",y="")+
    theme_classic()+theme(axis.text.y = element_blank(),axis.ticks.y = element_blank(),axis.line.y = element_blank())
  
  
  

  
  p3 <- ggplot(gdata.35)+
    #geom_rect(aes(xmin=V7, xmax=V8, ymin=id-1 ,ymax=id),fill="#0071bc")+
     geom_rect(aes(xmin=V13, xmax=V14, ymin=id-1 ,ymax=id),fill="#cc6666")+
    scale_x_continuous(limits = c(min_x.1,max_x.1))+
    labs(title = paste0(x," 3'-5'  chimera"),x="SARS-CoV-2",y="")+
    theme_classic()+theme(axis.text.y = element_blank(),axis.ticks.y = element_blank())
  
  p4 <- ggplot(gdata.35)+
    geom_rect(aes(xmin=V7, xmax=V8, ymin=id-1 ,ymax=id),fill="#6699cc")+
   # geom_rect(aes(xmin=V13, xmax=V14, ymin=id-1 ,ymax=id),fill="#c1272d")+
    scale_x_continuous(limits = c(min_x.2,max_x.2))+
    labs(x="SARS-CoV-2",y="")+
    theme_classic()+theme(axis.text.y = element_blank(),axis.ticks.y = element_blank(),axis.line.y = element_blank())
  
  p.list <- list(P1=p1,P2=p2,P3=p3,P4=p4)

}


matrix2cdt <- function(matrix,levels = (3000:0)*10,contrast = 10,cdtFileName){
  data.table <- data.table::data.table(bin1=factor(matrix$Arm5,levels = rev(levels)),
                                       bin2=factor(matrix$Arm3,levels = levels),
                                       n   =matrix$n/contrast)
  d <- data.table::dcast.data.table(data =data.table,formula = bin2~bin1,fill = 0,drop = F,value.var="n")
  
  d1 <- cbind(data.frame(COL_1 =d$bin2,COL_2=d$bin2,GWEIGHT=1),d[,2:ncol(d)]) %>% type.convert(as.is=F)
  d2 <- rbind(c("EWEIGHT",rep(1,ncol(d1)-1)),d1)
  
  data.table::fwrite(d2,file=cdtFileName,sep = "\t",quote = F)                                 
}




CPM2cdt <- function(samplelist,indir="matrix_all/",matxsufix="_R2_all.matrix.gz",outdir="cdt/",range=c(0,500),cutoff=0,cdtFileName,contrast=1,readcounts,...){
  
  # read raw matrix and total mapped readcount to make CPM matrix
  # then export to cdt
  #readcounts is a named vector, recording total read count of the sample
  suppressWarnings(library(ggplot2,quietly = T,warn.conflicts = F))
  suppressWarnings(library(dplyr,quietly = T,warn.conflicts = F))
  
  levels = 10*(floor(range[2]/10) : floor(range[1]/10))
  
  data <- lapply(samplelist, function(x){
    
    data <- data.table::fread(paste0(indir,x,matxsufix),sep = "\t",col.names = c("Arm5","Arm3","n"))
    data <- data %>% filter((Arm5 <= range[2] & Arm5 >= range[1]) &(Arm3 <= range[2] & Arm3 >= range[1]))
    data <- data %>% filter(n > cutoff)  %>% mutate(n=1e6*n/readcounts[x])
    
    matrix2cdt(matrix = data,levels =levels,contrast = contrast,cdtFileName = paste0(outdir,x,cdtFileName) )
    
  }) %>% bind_rows () %>% as.data.frame()
  
}



plot_local_mat_sym <- function(samplelist,indir="matrix_rm_SL/",matxsufix=".merged_R2_remove_selfligation.matrix.gz",range=c(0,500),cutoff=0,annotation=NULL,...){
 
  # this function is using contact matrix, to distill region of interest, and then make ggplots
  # only use 5'-3' chimeras, because they are more convincing
  suppressWarnings(library(ggplot2,quietly = T,warn.conflicts = F))
  suppressWarnings(library(dplyr,quietly = T,warn.conflicts = F))
  
  # first read contact matrix from each sample, and merged into a data.frame
   data <- lapply(samplelist, function(x){

    data <- data.table::fread(paste0(indir,x,matxsufix),sep = "\t",col.names = c("Arm5","Arm3","n"))
    data <- data %>% filter((Arm5 <= range[2] & Arm5 >= range[1]) &(Arm3 <= range[2] & Arm3 >= range[1]))
    data <- data %>% filter(n > cutoff) %>% mutate(sample=x)
  }) %>% bind_rows () %>% as.data.frame() %>% filter(Arm5 <= Arm3)
   
   # make a symetric matrix
  data.2 <- data.frame(Arm5=data$Arm3,Arm3=data$Arm5,n=data$n,sample=data$sample) %>% filter(Arm5 >Arm3)
  data <- rbind(data,data.2)
  # ceiling high outliers
  data$n[data$n > quantile(data$n,0.95)] <- quantile(data$n,0.95)
  maxpoint <- max(data$n)

  # make list of ggplots
  p.list <- lapply(samplelist,function(x){
    gdata <- data %>% filter(sample == x)
    p <- ggplot(gdata,aes(Arm5,Arm3))+
      geom_rect(aes(fill=n,xmin=Arm5,xmax=Arm5+10,ymin=Arm3,ymax=Arm3+10))+
      annotate(geom = "rect",xmin = range[1], xmax = range[2], ymin = range[1], ymax = range[2],fill=NA,color="black")+
      annotate(geom = "rect",xmin = floor(annotation$V2/10)*10, xmax = floor(1+annotation$V3/10)*10, ymin = floor(annotation$V4/10)*10, ymax = floor(1+annotation$V5/10)*10,fill=NA,color="black")+
      annotate(geom = "rect",ymin = floor(annotation$V2/10)*10, ymax = floor(1+annotation$V3/10)*10, xmin = floor(annotation$V4/10)*10, xmax = floor(1+annotation$V5/10)*10,fill=NA,color="black")+
      scale_fill_gradient(low = "#ffffff",high="#000000",limits=c(0,maxpoint))+
      labs(fill="chimeras",title = x)+
      theme_void()+theme(aspect.ratio = 1)

  })
}


plot_local_mat <- function(samplelist,indir="matrix_rm_SL/",matxsufix=".merged_R2_remove_selfligation.matrix.gz",range=c(0,500),cutoff=0,ceiling=0.95,annotation=NULL,...){
  suppressWarnings(library(ggplot2,quietly = T,warn.conflicts = F))
  suppressWarnings(library(dplyr,quietly = T,warn.conflicts = F))
  data <- lapply(samplelist, function(x){

    data <- data.table::fread(paste0(indir,x,matxsufix),sep = "\t",col.names = c("Arm5","Arm3","n"))
    data <- data %>% filter((Arm5 <= range[2] & Arm5 >= range[1]) &(Arm3 <= range[2] & Arm3 >= range[1]))
    data <- data %>% filter(n > cutoff) %>% mutate(sample=x)
  }) %>% bind_rows () %>% as.data.frame()
  
  
  data$n[data$n > quantile(data$n,ceiling)] <- quantile(data$n,ceiling)
  
  maxpoint <- max(data$n)
  regionMax <- max(data$Arm5,data$Arm3)
  regionMin <- min(data$Arm5,data$Arm3)
  
  p.list <- lapply(samplelist,function(x){
    gdata <- data %>% filter(sample == x)
    p <- ggplot(gdata,aes(Arm5,Arm3))+
      geom_rect(aes(fill=n,xmin=Arm5,xmax=Arm5+10,ymin=Arm3,ymax=Arm3+10))+
      annotate(geom = "rect",xmin = range[1]-5, xmax = range[2]-5, ymin = range[1]-5, ymax = range[2]-5,fill=NA,color="black")+ 
      
      annotate(geom = "rect",xmin = floor(annotation$V2/10)*10, xmax = floor(1+annotation$V3/10)*10, ymin = floor(annotation$V4/10)*10, ymax = floor(1+annotation$V5/10)*10,fill=NA,color="black")+
      annotate(geom = "rect",ymin = floor(annotation$V2/10)*10, ymax = floor(1+annotation$V3/10)*10, xmin = floor(annotation$V4/10)*10, xmax = floor(1+annotation$V5/10)*10,fill=NA,color="black")+
      scale_fill_gradient(low = "#ffffff",high="#000000",limits=c(0,maxpoint))+
      scale_x_continuous(breaks = c(regionMin,regionMax))+
      labs(fill="chimeras",title = x)+
      theme_bw()+
      theme(aspect.ratio = 1,panel.grid = element_blank())
  })
}

plot_local_CPM <- function(samplelist,indir="matrix_all/",matxsufix=".merged_R2_all.matrix.gz",range=c(0,500),cutoff=0,ceiling=0.95,annotation=NULL,readcounts,...){
  suppressWarnings(library(ggplot2,quietly = T,warn.conflicts = F))
  suppressWarnings(library(dplyr,quietly = T,warn.conflicts = F))
  data <- lapply(samplelist, function(x){
    
    data <- data.table::fread(paste0(indir,x,matxsufix),sep = "\t",col.names = c("Arm5","Arm3","n"))
    data <- data %>% filter((Arm5 <= range[2] & Arm5 >= range[1]) &(Arm3 <= range[2] & Arm3 >= range[1]))
    data <- data %>% filter(n > cutoff) %>% mutate(sample=x) %>% mutate(n=1e6*n/readcounts[x])
  }) %>% bind_rows () %>% as.data.frame()
  
  
  data$n[data$n > quantile(data$n,ceiling)] <- quantile(data$n,ceiling)
  
  maxpoint <- max(data$n)
  regionMax <- max(data$Arm5,data$Arm3)
  regionMin <- min(data$Arm5,data$Arm3)
  
  p.list <- lapply(samplelist,function(x){
    gdata <- data %>% filter(sample == x)
    p <- ggplot(gdata,aes(Arm5,Arm3))+
      geom_rect(aes(fill=n,xmin=Arm5,xmax=Arm5+10,ymin=Arm3,ymax=Arm3+10))+
      annotate(geom = "rect",xmin = range[1], xmax = range[2]+10, ymin = range[1], ymax = range[2]+10,fill=NA,color="black")+ 
      annotate(geom = "rect",xmin = floor(annotation$V2/10)*10, xmax = floor(1+annotation$V3/10)*10, ymin = floor(annotation$V4/10)*10, ymax = floor(1+annotation$V5/10)*10,fill=NA,color="black")+
      annotate(geom = "rect",ymin = floor(annotation$V2/10)*10, ymax = floor(1+annotation$V3/10)*10, xmin = floor(annotation$V4/10)*10, xmax = floor(1+annotation$V5/10)*10,fill=NA,color="black")+
      annotate(geom = "text",x=(annotation$V2+annotation$V3)/2,y=(annotation$V4+annotation$V5)/2,label=annotation$V1,color="#0071bc",size=2)+
      scale_fill_gradient(low = "#ffffff",high="#000000",limits=c(0,maxpoint))+
      scale_x_continuous(breaks = c(regionMin,regionMax),name = NULL)+
      labs(fill="chimeras per million reads",title = x)+
      theme_bw()+
      theme(aspect.ratio = 1,panel.grid = element_blank(),panel.border =  element_blank())
  })
}

plot_2region_mat <- function(samplelist,indir="matrix_rm_SL/",matxsufix=".merged_R2_remove_selfligation.matrix.gz",symetry=T,range1=c(0,500),range2=c(29500,29870),cutoff=0,ceiling=0.95,annotation=NULL,...){
  suppressWarnings(library(ggplot2,quietly = T,warn.conflicts = F))
  suppressWarnings(library(dplyr,quietly = T,warn.conflicts = F))
  data <- lapply(samplelist, function(x){
    
    data <- data.table::fread(paste0(indir,x,matxsufix),sep = "\t",col.names = c("Arm5","Arm3","n"))
    data.1 <- data %>% filter((Arm5 <= range1[2] & Arm5 >= range1[1]) &(Arm3 <= range2[2] & Arm3 >= range2[1]))
    data.2 <- data %>% filter((Arm5 <= range2[2] & Arm5 >= range2[1]) &(Arm3 <= range1[2] & Arm3 >= range1[1]))
    
    
    data <- rbind(data.1,data.2) %>% filter(n > cutoff) %>% mutate(sample=x)
  }) %>% bind_rows () %>% as.data.frame()
  
  #make symetry
  data.1 <- data.frame(Arm5=data$Arm5,Arm3=data$Arm3,n=data$n,sample=data$sample) 
  data.2 <- data.frame(Arm5=data$Arm3,Arm3=data$Arm5,n=data$n,sample=data$sample)
  data <- rbind(data.1,data.2) %>% mutate(bin1=pmin(Arm5,Arm3),bin2=pmax(Arm5,Arm3)) %>% group_by(bin1,bin2,sample) %>% summarise(n=sum(n)) %>%
    mutate(Arm5=bin1,Arm3=bin2)
  
  data$n[data$n > quantile(data$n,ceiling)] <- quantile(data$n,ceiling)
  
  
  
  maxpoint <- max(data$n)
  regionMax <- max(data$Arm5,data$Arm3)
  regionMin <- min(data$Arm5,data$Arm3)
  
  p.list <- lapply(samplelist,function(x){
    gdata <- data %>% filter(sample == x)
    p <- ggplot(gdata,aes(Arm5,Arm3))+
      geom_rect(aes(fill=n,xmin=Arm5,xmax=Arm5+10,ymin=Arm3,ymax=Arm3+10))+
      annotate(geom = "rect",xmin = range1[1]-5, xmax = range1[2]-5, ymin = range2[1]-5, ymax = range2[2]-5,fill=NA,color="black")+ 
      
      annotate(geom = "rect",xmin = floor(annotation$V2/10)*10, xmax = floor(1+annotation$V3/10)*10, ymin = floor(annotation$V4/10)*10, ymax = floor(1+annotation$V5/10)*10,fill=NA,color="black")+
      annotate(geom = "rect",ymin = floor(annotation$V2/10)*10, ymax = floor(1+annotation$V3/10)*10, xmin = floor(annotation$V4/10)*10, xmax = floor(1+annotation$V5/10)*10,fill=NA,color="black")+
      scale_fill_gradient(low = "#ffffff",high="#000000",limits=c(0,maxpoint))+
      
      labs(fill="chimeras",title = x)+
      theme_bw()+
      theme(aspect.ratio = 1,panel.grid = element_blank())
  })
}


# this function has bugs
# matrix cannot be called
plot_matrix_from_hyb <- function(indir="../Hyb(Greg)/",samplelist,
                                 suffix="_R2_comp_SARS-CoV-2_no_polyA_hybrids_ua.hyb.gz",
                                 Bait_RNA="NC_045512-2_SARS-CoV-2_virusRNA",overlapFilter=100,
                                 range=c(0,500),bin=1,pad=20,annotation=NULL,...){
  
  # only for intra region matrix
  library(dplyr)
  
  gdata <- lapply(samplelist,function(x){
    hybfile <- paste0(indir,x,suffix)
    # READ and filter
    # filter RNA
    hyblines <- data.table::fread(hybfile,sep = "\t") %>% filter(V4 == V10) %>% filter(V4 == Bait_RNA) %>%
      # reduce size
      select(c(7,8,13,14)) %>%
      # filter region
      mutate(V7=floor(V7/bin)*bin,V8=floor(V8/bin)*bin,V13=floor(V13/bin)*bin,V14=floor(V14/bin)*bin) %>%
      filter( V7 >= range[1] & V8 <= range[2]+pad & V13 >= range[1]-pad & V14 <= range[2]+pad) %>%
      
      # filter ligation overlap/distance
      mutate(overlap = 1+pmin(V8,V14)-pmax(V7,V13)) %>% filter(overlap < overlapFilter) %>% 
    
      # count 
      count(V7,V8,V13,V14)  %>% as.data.frame()
    
    hyblines[,1:4] <- floor(hyblines[,1:4] / bin)
    
    Arm5 <- Arm3 <- bin*(min(hyblines$V7,hyblines$V13,hyblines$V8,hyblines$V14) :max(hyblines$V7,hyblines$V13,hyblines$V8,hyblines$V14))
    len <- max(Arm5)/bin-min(Arm5)/bin+1
    matrix <- matrix(data = 0,nrow =len,ncol =len,dimnames=list(Arm5,Arm3))
    
    
    for(i in 1:nrow(hyblines)){
      matrix[(floor(hyblines[i,1]/bin):floor(hyblines[i,2]/bin))*bin,(floor(hyblines[i,3]/bin):floor(hyblines[i,4]/bin))*bin] <- matrix[(floor(hyblines[i,1]/bin):floor(hyblines[i,2]/bin))*bin,(floor(hyblines[i,3]/bin):floor(hyblines[i,4]/bin))*bin] +hyblines$n[i]
    }
    data <- reshape2::melt(matrix, varnames = c("Arm5", "Arm3"),value.name="n") %>% type.convert() %>% dplyr::filter(n>0) %>% mutate(sample=x)
    
  }) %>% bind_rows() %>% as.data.frame %>% mutate(n = ifelse(n < quantile(n,0.95),n,quantile(n,0.95)))
  
  maxcount <- max(gdata$n)
  regionMax <- max(gdata$Arm5,gdata$Arm3)
  regionMin <- min(gdata$Arm5,gdata$Arm3)
  
  p.list <- lapply(samplelist,function(x){
    gdata.x <- gdata %>% filter(sample==x) 
    p <- ggplot(gdata.x,aes(Arm5,Arm3))+
      geom_rect(aes(fill=n,xmin=Arm5,xmax=Arm5+bin,ymin=Arm3,ymax=Arm3+bin))+
      annotate(geom = "rect",xmin = regionMin, xmax = regionMax+bin, ymin = regionMin, ymax = regionMax+bin,fill=NA,color="black")+
      annotate(geom = "rect",xmin = annotation$V2, xmax = annotation$V3, ymin = annotation$V4, ymax = annotation$V5,fill=NA,color="black")+
      annotate(geom = "rect",ymin = annotation$V2, ymax = annotation$V3, xmin = annotation$V4, xmax = annotation$V5,fill=NA,color="black")+
      scale_fill_gradient(low = "#ffffff",high="#000000",limits=c(0,maxcount))+
      scale_x_continuous(breaks = c(regionMin,regionMax))+
      labs(fill="chimeras",title = x)+
      theme_bw()+
      theme(aspect.ratio = 1,panel.grid = element_blank())
  })
}

CPM2cdt_zoom <- function(samplelist,indir="matrix_all/",matxsufix="_R2_all.matrix.gz",outdir="cdt/",range_Arm5=c(0,500),range_Arm3=c(0,500),cutoff=0,cdtFileName,contrast=1,readcounts,...){
  
  # read raw matrix and total mapped readcount to make CPM matrix
  # then export to cdt
  #readcounts is a named vector, recording total read count of the sample
  suppressWarnings(library(ggplot2,quietly = T,warn.conflicts = F))
  suppressWarnings(library(dplyr,quietly = T,warn.conflicts = F))
  
  levels_5 = 10*(floor(range_Arm5[2]/10) : floor(range_Arm5[1]/10))
  levels_3 = 10*(floor(range_Arm3[1]/10) : floor(range_Arm3[2]/10))
  data <- lapply(samplelist, function(x){
    
    data <- data.table::fread(paste0(indir,x,matxsufix),sep = "\t",col.names = c("Arm5","Arm3","n"))
    data <- data %>% filter((Arm5 <= range_Arm5[2] & Arm5 >= range_Arm5[1]) &(Arm3 <= range_Arm3[2] & Arm3 >= range_Arm3[1]))
    matrix <- data %>% filter(n > cutoff)  %>% mutate(n=1e6*n/readcounts[x])
    
    
    
    data.table <- data.table::data.table(bin1=factor(matrix$Arm5,levels = levels_5),
                                         bin2=factor(matrix$Arm3,levels = levels_3),
                                         n   =matrix$n/contrast) 
    
    d <- data.table::dcast.data.table(data =data.table,formula = bin2~bin1,fill = 0,drop = F,value.var="n")
    
    d1 <- cbind(data.frame(COL_1 =d$bin2,COL_2=d$bin2,GWEIGHT=1),d[,2:ncol(d)]) %>% type.convert()
    d2 <- rbind(c("EWEIGHT",rep(1,ncol(d1)-1)),d1)
    
    data.table::fwrite(d2,file=paste0(outdir,x,cdtFileName),sep = "\t",quote = F)    
    
    
    
  })
  
}

# correaltion between two matrix
cor_matrix <- function(x1,x2,range=c(0,30000),col_n=3){
  mat1 <- data.table::fread(file = x1,sep = "\t",select = c(1,2,col_n),col.names = c("Arm5","Arm3","n")) %>% 
    filter(Arm5 <= range[2] & Arm5 >= range[1] &Arm3 <= range[2] & Arm3 >= range[1]) %>% as.data.frame()
  mat2 <- data.table::fread(file = x2,sep = "\t",select = c(1,2,col_n),col.names = c("Arm5","Arm3","n")) %>%
    filter(Arm5 <= range[2] & Arm5 >= range[1] &Arm3 <= range[2] & Arm3 >= range[1]) %>% as.data.frame()
  levels <- 10*(floor(range[1]/10):floor(range[2]/10) )
  mat1 <- data.table::data.table(Arm5=factor(mat1$Arm5,levels = levels),Arm3=factor(mat1$Arm3,levels = levels),n=mat1$n) %>%
    data.table::dcast.data.table(formula = Arm5~Arm3,fill=0,drop = F,value.var = "n") %>% data.table::melt.data.table(id.vars = "Arm5",variable.name = "Arm3",value.name = "n")
  
  mat2 <- data.table::data.table(Arm5=factor(mat2$Arm5,levels = levels),Arm3=factor(mat2$Arm3,levels = levels),n=mat2$n) %>%
    data.table::dcast.data.table(formula = Arm5~Arm3,fill=0,drop = F,value.var = "n") %>% data.table::melt.data.table(id.vars = "Arm5",variable.name = "Arm3",value.name = "n")
  
  data <- data.frame(mat1=mat1$n,mat2=mat2$n) %>% filter(mat1+mat2 >0)
 # t1 <- cor.test(data$mat1,data$mat2)
  t2 <- cor.test(mat1$n,mat2$n)
  return(t2)
}




# turn COMRADE cluster to bin-bin pair

cluster2CM <- function(sheet = "SARS-CoV-2_cluster_Sample1",binsize = 10){
  Cluster_gRNA_S1 <- read_excel("/Volumes/HiR/SARS2-HiR/publicData/1-s2.0-S1097276520307826-mmc3(clusters).xlsx", 
                                sheet = sheet)
  hyblines <- Cluster_gRNA_S1[,2:5]
  CM <- hyb2CM(hyblines =hyblines, binsize =binsize) %>% mutate(bin1=pmin(Arm5,Arm3),bin2=pmax(Arm5,Arm3)) %>% count(bin1,bin2) 
}


# blast coverage

# core function
blast_cov <- function(blastlines,genomeLen,...) {
  
  genome.cov <- as.numeric(rep(0,genomeLen))
  for(i in 1:nrow(blastlines)){
    genome.cov[as.integer(blastlines[i,3]):as.integer(blastlines[i,4])] <- genome.cov[as.integer(blastlines[i,3]):as.integer(blastlines[i,4])]  +1
  }
  data <- data.frame(pos=1:genomeLen,genome_cov=genome.cov)
  
}

# for dupliated blast
blast_cov_dup <- function(blastlines,...) {
  
  
  genome.cov <- as.numeric(rep(0,genomeLen))
  for(i in 1:nrow(blastlines)){
    genome.cov[as.integer(blastlines[i,4]):as.integer(blastlines[i,5])] <- genome.cov[as.integer(blastlines[i,4]):as.integer(blastlines[i,5])]  +as.integer(blastlines[i,2])
  }
  data <- data.frame(pos=1:genomeLen,genome_cov=genome.cov)
  
}


#blastlines <- data.table::fread("../hyb_PEAR/L1/L1.PEAR.fastp_comp_GX002_hOH7.gz_mtophits.blast",nrows = 100,sep = "\t",select = c(1:2,9:11)) %>% tidyr::separate(into =c("id","count"), col = "V1",sep="_")


# parallel function
runparallel <- function(objlist, FUN, cores = 7, ...) {
  
  cl <- parallel::makeCluster(cores)
  res <- parallel::parLapply(cl, X = objlist, fun = FUN, ...)
  parallel::stopCluster(cl)
  
  invisible(res)
}

mytime <- function(expression) {
  t1 <- proc.time()
  res <- eval(expression)
  print(proc.time() - t1)
  res
}



# package function
BlastToCov <- function(Blastfile,out_filename,RNA="GX002_vius_HIV_virusRNA",genomeLen,...){
  library(dplyr)
  
  Blast <- data.table::fread(Blastfile,sep = "\t",select = c(1:2,9:11))
  Blast <- Blast %>% dplyr::filter(V2 == RNA) 
  cores <- parallel::detectCores() -1
  idxes <- sort(sample(letters[1:cores], nrow(Blast), replace = TRUE))
  BlastList <- split(Blast, idxes)
  result <- mytime({
    runparallel(BlastList, FUN = blast_cov, cores = cores,genomeLen=genomeLen)
  })%>% bind_rows()  %>% group_by(pos) %>% summarise(n = sum(genome_cov)) %>% as.data.frame()
  data.table::fwrite(result,file = out_filename,row.names = F,sep = "\t")
  
}

BlastToCov_dup <- function(Blastfile,out_filename,RNA="GX002_vius_HIV_virusRNA",...){
  library(dplyr)
  
  Blast <- data.table::fread(Blastfile,sep = "\t",select = c(1:2,9:11))
  
  Blast <- Blast %>% dplyr::filter(V2 == RNA) 
  Blast <- Blast %>% tidyr::separate(into =c("id","count"), col = "V1",sep="_")
  
  cores <- parallel::detectCores() -1
  idxes <- sort(sample(letters[1:cores], nrow(Blast), replace = TRUE))
  BlastList <- split(Blast, idxes)
  result <- mytime({
    runparallel(BlastList, FUN = blast_cov_dup, cores = cores)
  })%>% bind_rows()  %>% group_by(pos) %>% summarise(n = sum(genome_cov)) %>% as.data.frame()
  data.table::fwrite(result,file = out_filename,row.names = F,sep = "\t")
  
}



VP <- function(bait=c(5000,5500),contact.matrix=CPM.CM){
  
  
  baited <- contact.matrix %>% filter(((Arm5 <= bait[2]) &(Arm5 >= bait[1]))|((Arm3 <= bait[2]) &(Arm3 >= bait[1]))) %>% 
    mutate(bait=ifelse(test = (Arm5 <= bait[2] & Arm5 >= bait[1]),yes = Arm5,no = Arm3),
           target=ifelse(test = (Arm5 <= bait[2] & Arm5 >= bait[1]),yes = Arm3,no = Arm5)) %>% 
    dplyr::group_by(target) %>% summarise(cpm.CM=sum(cpm.CM))
}



# CT structure MDS
CT_MDS <- function(x=CTs) {
  
  d <- as.dist(1-cor(x,method = "spearman"))
  MDS <- cmdscale(d,eig = T,x.ret = T)
  MDS.df <- MDS$points %>% as.data.frame() %>% tibble::rownames_to_column(var = "ct")
  
  
  MDS.df$group <-  stringr::str_split(MDS.df$ct,pattern = "_",simplify = T,n = 2)[,1]
  p1 <- ggplot(MDS.df,aes(x = V1,y=V2,color=group))+
    geom_point()+
    geom_text_repel(aes(label=ct))+
    scale_color_manual(values = col_sample)+
    scale_x_continuous(name = paste0("PC1 ",100*MDS$GOF[1],"%"))+
    scale_y_continuous(name = paste0("PC2 ",100*(MDS$GOF[2]-MDS$GOF[1]),"%"))+
    theme_classic()+theme(aspect.ratio = 1)
  
}
