#!/bin/bash
# this is a script to find base pair score of each base pair in given .CT structure

# example :
# colorCT.sh -i GX002-7625-8062.ct -b 7625 -e 8062 -d ~/Documents/scientific/HIV-HiR/2nd/R/intermediated/COMRADES_score_merged -s GC.merged_score.txt.gz -a GC -c 4 

# parsing command line options
while getopts "i:b:e:s:d:a:c:" OPTION
do
     case $OPTION in
         i) INPUT_CT=$OPTARG ;;
         b) BEGIN=$OPTARG ;;
         e) END=$OPTARG ;;
		 s) BASEPAIR_SCORES=$OPTARG ;;
         d) Dir_basepair_score=$OPTARG ;;
		 a) Alias_of_basepair_score=$OPTARG ;;
		 c) Cloumn_score=$OPTARG ;;
         ?)
             echo "incorrect option"
             exit 0
             ;;
     esac
done

BASEPAIR_FULLPATH=${Dir_basepair_score}/${BASEPAIR_SCORES}

CORLORMAP=${INPUT_CT/.ct/}".colormap."${BASEPAIR_SCORES}".txt"

modifed_CT=${INPUT_CT/.ct/.modifed.ct}


FRAGMENT_SCORES=${BEGIN}_${END}".fragment_scores.txt"



if [[ $(file ${BASEPAIR_FULLPATH}) == *gzip* ]]; then
    echo "The basepair score is gzipped."
    cat ${BASEPAIR_FULLPATH} |zcat | awk -v b=$BEGIN -v e=$END -v c=${Cloumn_score} 'BEGIN {FS="\t"; OFS="\t"};NR>1 && $1>=b && $1<=e && $2>=b && $2<=e && $c>0{print $1,$2,$c}'  > $FRAGMENT_SCORES

else
    echo "The basepair score is not  gzipped."
    cat ${BASEPAIR_FULLPATH} | awk -v b=$BEGIN -v e=$END -v c=${Cloumn_score} 'BEGIN {FS="\t"; OFS="\t"};NR>1 && $1>=b && $1<=e && $2>=b && $2<=e && $c>0{print $1,$2,$c}'  > $FRAGMENT_SCORES

fi



# calculate score
CORLORMAP=${INPUT_CT/.ct/}".colormap."${Alias_of_basepair_score}".txt"
modifed_CT=${INPUT_CT/.ct/.mod.ct}
rm -f $CORLORMAP 


awk -v B=${BEGIN} 'BEGIN{OFS="\t"; FS="\t"} !/dG/  {print $1+B-1,$5+B-1}' $INPUT_CT > temp.ct
cat temp.ct | while read CT_bit1 CT_bit2; do
    awk -v CT_bit1=${CT_bit1} -v CT_bit2=${CT_bit2} -v B=${BEGIN} -f ~/src/scoreCT.awk $FRAGMENT_SCORES >> $CORLORMAP
done 

Total_score=`awk 'BEGIN{total=0;FS="\t"} {total=total+$2}END{print total}' $CORLORMAP`
awk -v B=${BEGIN} -v Total_score=${Total_score} 'BEGIN{OFS="\t";FS="\t"} /dG/{print $0,Total_score};!/dG/{print $1,$2,$3,$4,$5,$1+B-1,$7,$8}' $INPUT_CT > $modifed_CT


rm -f temp.ct 